﻿using System;
using System.Threading;
using System.Collections.Generic;

using wclCommon;
using wclWiFi;

namespace WiFiDirectClient
{
    class Program
    {
        private class WiFiWatcher
        {
            private ManualResetEvent CompleteEvent;
            private Thread Thread;

            private void wclWiFiDirectDeviceWatcherDiscoveringStarted(Object Sender, EventArgs e)
            {
                Console.WriteLine("Discovering started");
            }

            private void wclWiFiDirectDeviceWatcherDeviceFound(Object Sender, String Id, String Name)
            {
                Console.Write("Found device " + Name);
                Devices.Add(Id);
            }

            private void wclWiFiDirectDeviceWatcherDiscoveringCompleted(Object Sender, Int32 Error)
            {
                Console.WriteLine("Discovering completed with result: 0x" + Error.ToString("X8"));
                CompleteEvent.Set();
            }

            private void ThreadProc()
            {
                wclWiFiDirectDeviceWatcher Watcher = new wclWiFiDirectDeviceWatcher();
                Watcher.OnDeviceFound += wclWiFiDirectDeviceWatcherDeviceFound;
                Watcher.OnDiscoveringCompleted += wclWiFiDirectDeviceWatcherDiscoveringCompleted;
                Watcher.OnDiscoveringStarted += wclWiFiDirectDeviceWatcherDiscoveringStarted;

                Console.WriteLine("Try to find WiFi Direct devices");
                Int32 Res = Watcher.Discover();
                if (Res != wclErrors.WCL_E_SUCCESS)
                    Console.WriteLine("Failed to start WiFi Direct device watcher: 0x" + Res.ToString("X8"));
                else
                    wclMessageBroadcaster.Wait(CompleteEvent);
            }

            public WiFiWatcher()
            {
                CompleteEvent = null;
                Devices = null;
                Thread = null;
            }

            public List<String> Devices;

            public void Discover()
            {
                CompleteEvent = new ManualResetEvent(false);
                Devices = new List<String>();
                Thread = new Thread(ThreadProc);
                Thread.Start();
                Thread.Join();
                CompleteEvent.Close();
            }
        };

        private class WiFiClient
        {
            private String Id;
            private ManualResetEvent TerminateEvent;
            private Thread Thread;

            private void wclWiFiDirectClientDeviceConnected(Object Sender, wclWiFiDirectDevice Device,
                Int32 Error)
            {
                if (Error != wclErrors.WCL_E_SUCCESS)
                    Console.WriteLine("Device " + Device.Name + " connect error: 0x" + Error.ToString("X8"));
                else
                {
                    Console.WriteLine("Device " + Device.Name + " has been connected");
                    Console.WriteLine("  Local IP: " + Device.LocalAddress);
                    Console.WriteLine("  Remote IP: " + Device.RemoteAddress);
                }
            }

            private void wclWiFiDirectClientDeviceDisconnected(Object Sender, wclWiFiDirectDevice Device,
                Int32 Reason)
            {
                Console.WriteLine("Device " + Device.Name + " has been disconnected by reason: 0x" +
                    Reason.ToString("X8"));
            }

            private void wclWiFiDirectClientPairCompleted(Object Sender, wclWiFiDirectDevice Device, Int32 Result)
            {
                Console.WriteLine("Pairing with device " + Device.Name + " has been completed with result: 0x" +
                    Result.ToString("X8"));
            }

            private void wclWiFiDirectClientPairConfirm(Object Sender, wclWiFiDirectDevice Device,
                out Boolean Confirm)
            {
                Console.WriteLine("Pairing with device " + Device.Name + " has been confirmed");
                Confirm = true;
            }

            private void wclWiFiDirectClientPairDisplayPin(Object Sender, wclWiFiDirectDevice Device, String Pin)
            {
                Console.WriteLine("To pair with device " + Device.Name + " type this PIN: " + Pin +
                    " on device keyboard");
            }

            private void wclWiFiDirectClientPairGetParams(Object Sender, wclWiFiDirectDevice Device,
                out Int16 GroupOwnerIntent, out wclWiFiDirectConfigurationMethod ConfigurationMethods,
                out wclWiFiDirectPairingProcedure PairingProcedure)
            {
                Console.WriteLine("Connection is accepted with following parameters:");
                Console.WriteLine("  Pairing procedure: Invitation");
                Console.WriteLine("  GroupOwnerIntent: default (14)");
                Console.WriteLine("  Configuration (pairing) method: Push Button (Configm Only)");

                PairingProcedure = wclWiFiDirectPairingProcedure.ppInvitation;
                GroupOwnerIntent = 14;
                ConfigurationMethods = wclWiFiDirectConfigurationMethod.cmPushButton;
            }

            private void wclWiFiDirectClientPairProvidePin(Object Sender, wclWiFiDirectDevice Device,
                out String Pin)
            {
                Console.WriteLine("Device requires pairing with entering PIN on Advertiser side. This type of pairing is not supported in this demo");
                Pin = "";
            }

            private void ThreadProc()
            {
                wclWiFiDirectClient Client = new wclWiFiDirectClient();
                Client.OnDeviceConnected += wclWiFiDirectClientDeviceConnected;
                Client.OnPairCompleted += wclWiFiDirectClientPairCompleted;
                Client.OnPairConfirm += wclWiFiDirectClientPairConfirm;
                Client.OnPairDisplayPin += wclWiFiDirectClientPairDisplayPin;
                Client.OnPairProvidePin += wclWiFiDirectClientPairProvidePin;
                Client.OnDeviceDisconnected += wclWiFiDirectClientDeviceDisconnected;
                Client.OnPairGetParams += wclWiFiDirectClientPairGetParams;

                Int32 Res = Client.Connect(Id);
                if (Res != wclErrors.WCL_E_SUCCESS)
                    Console.WriteLine("Connect failed: 0x" + Res.ToString("X8"));

                wclMessageBroadcaster.Wait(TerminateEvent);

                if (Client.State != wclWiFiDirectDeviceState.dsDisconnected)
                {
                    Res = Client.Disconnect();
                    if (Res != wclErrors.WCL_E_SUCCESS)
                        Console.WriteLine("Disconnect failed: 0x" + Res.ToString("X8"));
                }
                Client = null;
            }

            public WiFiClient()
            {
                TerminateEvent = null;
                Thread = null;
                Id = "";
            }

            public void Connect(String _Id)
            {
                TerminateEvent = new ManualResetEvent(false);
                Id = _Id;
                Thread = new Thread(ThreadProc);
                Thread.Start();
            }

            public void Disconnect()
            {
                TerminateEvent.Set();
                Thread.Join();
                Thread = null;
                TerminateEvent.Close();
                TerminateEvent = null;
                Id = "";
            }
        };

        static void Main(string[] args)
        {
            wclMessageBroadcaster.SetSyncMethod(wclMessageSynchronizationKind.skApc);

            Console.Clear();
            Console.WriteLine("WIFI DIRECT CLIENT DEMO");
            Console.WriteLine("=======================");
            Console.WriteLine("The demo shows how to discover WiFi Direct devices and connect to one.");
            Console.WriteLine("This demo will connect to first found WiFi Direct device");

            WiFiWatcher Watcher = new WiFiWatcher();
            Watcher.Discover();

            Int32 Ndx = -1;
            if (Watcher.Devices == null || Watcher.Devices.Count == 0)
                Console.WriteLine("No WiFi Direct devices were found");
            else
            {
                Int32 n = 0;
                foreach (String Id in Watcher.Devices)
                {
                    Console.WriteLine(n.ToString() + ": " + Id);
                    n++;
                }
                Console.Write("Select device: ");
                String ns = Console.ReadLine();
                Ndx = Convert.ToInt32(ns);
            }

            WiFiClient Client = null;
            if (Ndx > -1)
            {
                Client = new WiFiClient();
                Client.Connect(Watcher.Devices[Ndx]);
            }

            Console.WriteLine("PRESS ANY KEY TO EXIT");
            Console.ReadKey();

            if (Client != null)
            { 
                Client.Disconnect();
                Client = null;
            }

            Watcher = null;
        }
    };
}
